@@ -32,17 +32,18 @@
 #define FILENAME_MAX_LENGTH 30 /* FIXME: find out the best value */
 
 #define get_home_relative_dir(x)        \
-        g_strconcat (g_get_home_dir (), \
-                     "/",               \
-                     (x),               \
-                     NULL)
+	g_strconcat (g_get_home_dir (), \
+		     "/",               \
+		     (x),               \
+		     NULL)
 
 gboolean            path_is_file                 (const gchar *s);
 gboolean            path_exists                  (const gchar *s);
+gboolean            uri_exists                   (const char  *uri);
 gboolean            path_is_dir                  (const gchar *s);
 gboolean            dir_is_empty                 (const gchar *s);
-gboolean            dir_contains_one_object      (const char  *path);
-gboolean            dir_is_temp_dir              (const char  *dir);
+gboolean            dir_contains_one_object      (const char  *uri);
+char *              get_directory_content_if_unique (const char  *uri);
 gboolean            path_in_path                 (const char  *path_src,
 						  const char  *path_dest);
 GnomeVFSFileSize    get_file_size                (const gchar *s);
@@ -69,8 +70,11 @@ gboolean            is_mime_type                 (const char* type,
 						  const char* pattern);
 G_CONST_RETURN char*get_mime_type                (const char  *filename);
 GnomeVFSFileSize    get_dest_free_space          (const char  *path);
-gboolean            rmdir_recursive              (const gchar *directory);
+gboolean            remove_directory             (const char  *uri);
+gboolean            remove_local_directory       (const char  *directory);
+GnomeVFSResult      make_tree                    (const char  *uri);
 char *              get_temp_work_dir            (void);
+gboolean            is_temp_work_dir             (const char *dir);
 char *              escape_uri                   (const char *uri);
 
 /* misc functions used to parse a command output lines. */
@@ -89,7 +93,22 @@ gboolean            check_permissions            (const char *path,
 						  int         mode);
 gboolean 	    is_program_in_path		 (const char *filename);
 
+/* URI/Path utils */
+
+const char *        get_home_uri                 (void);
 GnomeVFSURI *       new_uri_from_path            (const char *path);
+char *              get_uri_from_local_path      (const char *local_path);
+char *              get_local_path_from_uri      (const char *uri);
+gboolean            uri_has_scheme               (const char *uri);
 gboolean            uri_is_local                 (const char *uri);
+const char *        remove_host_from_uri         (const char *uri);
+char *              get_uri_host                 (const char *uri);
+char *              get_uri_root                 (const char *uri);
+char *              get_uri_from_path            (const char *path);
+int                 uricmp                       (const char *path1,
+						  const char *path2);
+char *              get_new_uri                  (const char *folder,
+	     					  const char *name);
+char *              get_new_uri_from_uri         (const char *uri);
 
 #endif /* FILE_UTILS_H */
