@@ -53,6 +53,8 @@ mktime_from_string (char *month,
 	struct tm     tm = {0, };
 	char        **fields;
 
+	tm.tm_isdst = -1;
+
 	/* date */
 
 	if (month != NULL) {
@@ -79,7 +81,7 @@ mktime_from_string (char *month,
 
 		fields = g_strsplit (time_or_year, ":", 2);
 		if (fields[0] != NULL) {
-			tm.tm_hour = atoi (fields[0]) - 1;
+			tm.tm_hour = atoi (fields[0]);
 			if (fields[1] != NULL) 
 				tm.tm_min = atoi (fields[1]);
 		}
@@ -203,7 +205,7 @@ process_line (char     *line,
 	if (*fdata->name == 0)
 		file_data_free (fdata);
 	else
-		comm->file_list = g_list_prepend (comm->file_list, fdata);
+		fr_command_add_file (comm, fdata);
 }
 
 
