@@ -30,7 +30,7 @@
 #include "file-list.h"
 #include "fr-archive.h"
 
-#define GCONF_NOTIFICATIONS 9
+#define GCONF_NOTIFICATIONS 10
 
 enum {
 	COLUMN_FILE_DATA,
@@ -44,6 +44,14 @@ enum {
 	NUMBER_OF_COLUMNS
 };
 
+enum {
+	TREE_COLUMN_PATH,
+	TREE_COLUMN_ICON,
+	TREE_COLUMN_NAME,
+	TREE_COLUMN_WEIGHT,
+	TREE_NUMBER_OF_COLUMNS
+};
+
 typedef enum {
 	FR_BATCH_ACTION_NONE,
 	FR_BATCH_ACTION_LOAD,
@@ -62,23 +70,6 @@ typedef enum {
 	FR_BATCH_ACTIONS
 } FRBatchActionType;
 
-typedef struct {
-	FRBatchActionType type;
-	void *            data;
-	GFreeFunc         free_func;
-} FRBatchAction;
-
-typedef struct {
-	guint      converting : 1;
-	char      *temp_dir;
-	FrArchive *new_archive;
-} FRConvertData;
-
-typedef enum {
-	FR_CLIPBOARD_OP_CUT,
-	FR_CLIPBOARD_OP_COPY
-} FRClipboardOp;
-
 /* -- FrWindow -- */
 
 #define FR_TYPE_WINDOW              (fr_window_get_type ())
@@ -123,6 +114,7 @@ FrWindow *  fr_window_archive_open              (FrWindow      *window,
 						 GtkWindow     *parent);
 void        fr_window_archive_close             (FrWindow      *window);
 const char *fr_window_get_archive_uri           (FrWindow      *window);
+const char *fr_window_get_paste_archive_uri     (FrWindow      *window);
 gboolean    fr_window_archive_is_present        (FrWindow      *window);
 void        fr_window_archive_save_as           (FrWindow      *window,
 						 const char    *filename);
@@ -176,6 +168,8 @@ void        fr_window_archive_extract_here      (FrWindow      *window,
 void        fr_window_archive_test	        (FrWindow      *window);					 
 void        fr_window_set_password              (FrWindow      *window,
 						 const char    *password);
+void        fr_window_set_password_for_paste    (FrWindow      *window,
+					         const char    *password);						 
 const char *fr_window_get_password              (FrWindow      *window);
 FRCompression fr_window_get_compression 	(FrWindow      *window);
 void        fr_window_view_folder_after_extract (FrWindow      *window,
@@ -262,6 +256,8 @@ void        fr_window_set_toolbar_visibility      (FrWindow   *window,
 						   gboolean    value);
 void        fr_window_set_statusbar_visibility    (FrWindow   *window,
 						   gboolean    value);
+void        fr_window_set_folders_visibility      (FrWindow   *window,
+						   gboolean    value);
 
 /* batch mode procedures. */
 
